@@ -5,55 +5,53 @@
 
 // spell-checker:ignore (clap) dont
 // spell-checker:ignore (ToDO) formatteriteminfo inputdecoder inputoffset mockstream nrofbytes partialreader odfunc multifile exitcode
-// spell-checker:ignore Anone
+// spell-checker:ignore Anone bfloat
 
 mod byteorder_io;
-mod formatteriteminfo;
-mod inputdecoder;
-mod inputoffset;
+mod formatter_item_info;
+mod input_decoder;
+mod input_offset;
 #[cfg(test)]
 mod mockstream;
-mod multifilereader;
+mod multifile_reader;
 mod output_info;
 mod parse_formats;
 mod parse_inputs;
 mod parse_nrofbytes;
-mod partialreader;
-mod peekreader;
+mod partial_reader;
+mod peek_reader;
 mod prn_char;
 mod prn_float;
 mod prn_int;
 
 use std::cmp;
 use std::fmt::Write;
-use std::io::BufReader;
+use std::io::{BufReader, Read};
 
 use crate::byteorder_io::ByteOrder;
-use crate::formatteriteminfo::FormatWriter;
-use crate::inputdecoder::{InputDecoder, MemoryDecoder};
-use crate::inputoffset::{InputOffset, Radix};
-use crate::multifilereader::{HasError, InputSource, MultifileReader};
+use crate::formatter_item_info::FormatWriter;
+use crate::input_decoder::{InputDecoder, MemoryDecoder};
+use crate::input_offset::{InputOffset, Radix};
+use crate::multifile_reader::{HasError, InputSource, MultifileReader};
 use crate::output_info::OutputInfo;
 use crate::parse_formats::{ParsedFormatterItemInfo, parse_format_flags};
 use crate::parse_inputs::{CommandLineInputs, parse_inputs};
 use crate::parse_nrofbytes::parse_number_of_bytes;
-use crate::partialreader::PartialReader;
-use crate::peekreader::{PeekRead, PeekReader};
+use crate::partial_reader::PartialReader;
+use crate::peek_reader::{PeekRead, PeekReader};
 use crate::prn_char::format_ascii_dump;
 use clap::ArgAction;
 use clap::{Arg, ArgMatches, Command, parser::ValueSource};
 use uucore::display::Quotable;
 use uucore::error::{UResult, USimpleError};
+use uucore::translate;
+
 use uucore::parser::parse_size::ParseSizeError;
 use uucore::parser::shortcut_value_parser::ShortcutValueParser;
-use uucore::{format_usage, help_about, help_section, help_usage, show_error, show_warning};
+use uucore::{format_usage, show_error, show_warning};
 
 const PEEK_BUFFER_SIZE: usize = 4; // utf-8 can be 4 bytes
 
-const ABOUT: &str = help_about!("od.md");
-const USAGE: &str = help_usage!("od.md");
-const AFTER_HELP: &str = help_section!("after help", "od.md");
-
 pub(crate) mod options {
     pub const HELP: &str = "help";
     pub const ADDRESS_RADIX: &str = "address-radix";
@@ -78,6 +76,21 @@ struct OdOptions {
     line_bytes: usize,
     output_duplicates: bool,
     radix: Radix,
+    string_min_length: Option<usize>,
+}
+
+/// Helper function to parse bytes with error handling
+fn parse_bytes_option(matches: &ArgMatches, option_name: &str) -> UResult<Option<u64>> {
+    match matches.get_one::<String>(option_name) {
+        None => Ok(None),
+        Some(s) => match parse_number_of_bytes(s) {
+            Ok(n) => Ok(Some(n)),
+            Err(e) => Err(USimpleError::new(
+                1,
+                format_error_message(&e, s, option_name),
+            )),
+        },
+    }
 }
 
 impl OdOptions {
@@ -89,7 +102,7 @@ impl OdOptions {
                 _ => {
                     return Err(USimpleError::new(
                         1,
-                        format!("Invalid argument --endian={s}"),
+                        translate!("od-error-invalid-endian", "endian" => s),
                     ));
                 }
             }
@@ -97,23 +110,12 @@ impl OdOptions {
             ByteOrder::Native
         };
 
-        let mut skip_bytes = match matches.get_one::<String>(options::SKIP_BYTES) {
-            None => 0,
-            Some(s) => match parse_number_of_bytes(s) {
-                Ok(n) => n,
-                Err(e) => {
-                    return Err(USimpleError::new(
-                        1,
-                        format_error_message(&e, s, options::SKIP_BYTES),
-                    ));
-                }
-            },
-        };
+        let mut skip_bytes = parse_bytes_option(matches, options::SKIP_BYTES)?.unwrap_or(0);
 
         let mut label: Option<u64> = None;
 
         let parsed_input = parse_inputs(matches)
-            .map_err(|e| USimpleError::new(1, format!("Invalid inputs: {e}")))?;
+            .map_err(|e| USimpleError::new(1, translate!("od-error-invalid-inputs", "msg" => e)))?;
         let input_strings = match parsed_input {
             CommandLineInputs::FileNames(v) => v,
             CommandLineInputs::FileAndOffset((f, s, l)) => {
@@ -149,23 +151,25 @@ impl OdOptions {
             cmp::max(max, next.formatter_item_info.byte_size)
         });
         if line_bytes == 0 || line_bytes % min_bytes != 0 {
-            show_warning!("invalid width {line_bytes}; using {min_bytes} instead");
+            show_warning!(
+                "{}",
+                translate!("od-error-invalid-width", "width" => line_bytes, "min" => min_bytes)
+            );
             line_bytes = min_bytes;
         }
 
         let output_duplicates = matches.get_flag(options::OUTPUT_DUPLICATES);
 
-        let read_bytes = match matches.get_one::<String>(options::READ_BYTES) {
+        let read_bytes = parse_bytes_option(matches, options::READ_BYTES)?;
+
+        let string_min_length = match parse_bytes_option(matches, options::STRINGS)? {
             None => None,
-            Some(s) => match parse_number_of_bytes(s) {
-                Ok(n) => Some(n),
-                Err(e) => {
-                    return Err(USimpleError::new(
-                        1,
-                        format_error_message(&e, s, options::READ_BYTES),
-                    ));
-                }
-            },
+            Some(n) => Some(usize::try_from(n).map_err(|_| {
+                USimpleError::new(
+                    1,
+                    translate!("od-error-argument-too-large", "option" => "-S", "value" => n.to_string()),
+                )
+            })?),
         };
 
         let radix = match matches.get_one::<String>(options::ADDRESS_RADIX) {
@@ -186,16 +190,13 @@ impl OdOptions {
                         _ => {
                             return Err(USimpleError::new(
                                 1,
-                                "Radix must be one of [o, d, x, n]".to_string(),
+                                translate!("od-error-radix-invalid", "radix" => s),
                             ));
                         }
                     }
                 } else {
                     // Return an error instead of panicking when `od -A ''` is executed.
-                    return Err(USimpleError::new(
-                        1,
-                        "Radix cannot be empty, and must be one of [o, d, x, n]".to_string(),
-                    ));
+                    return Err(USimpleError::new(1, translate!("od-error-radix-empty")));
                 }
             }
         };
@@ -210,6 +211,7 @@ impl OdOptions {
             line_bytes,
             output_duplicates,
             radix,
+            string_min_length,
         })
     }
 }
@@ -222,41 +224,52 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
 
     let clap_opts = uu_app();
 
-    let clap_matches = clap_opts.try_get_matches_from(&args)?;
+    let clap_matches = uucore::clap_localization::handle_clap_result(clap_opts, &args)?;
 
     let od_options = OdOptions::new(&clap_matches, &args)?;
 
-    let mut input_offset =
-        InputOffset::new(od_options.radix, od_options.skip_bytes, od_options.label);
+    // Check if we're in strings mode
+    if let Some(min_length) = od_options.string_min_length {
+        extract_strings_from_input(
+            &od_options.input_strings,
+            od_options.skip_bytes,
+            od_options.read_bytes,
+            min_length,
+            od_options.radix,
+        )
+    } else {
+        let mut input_offset =
+            InputOffset::new(od_options.radix, od_options.skip_bytes, od_options.label);
 
-    let mut input = open_input_peek_reader(
-        &od_options.input_strings,
-        od_options.skip_bytes,
-        od_options.read_bytes,
-    );
-    let mut input_decoder = InputDecoder::new(
-        &mut input,
-        od_options.line_bytes,
-        PEEK_BUFFER_SIZE,
-        od_options.byte_order,
-    );
+        let mut input = open_input_peek_reader(
+            &od_options.input_strings,
+            od_options.skip_bytes,
+            od_options.read_bytes,
+        );
+        let mut input_decoder = InputDecoder::new(
+            &mut input,
+            od_options.line_bytes,
+            PEEK_BUFFER_SIZE,
+            od_options.byte_order,
+        );
 
-    let output_info = OutputInfo::new(
-        od_options.line_bytes,
-        &od_options.formats[..],
-        od_options.output_duplicates,
-    );
+        let output_info = OutputInfo::new(
+            od_options.line_bytes,
+            &od_options.formats[..],
+            od_options.output_duplicates,
+        );
 
-    odfunc(&mut input_offset, &mut input_decoder, &output_info)
+        odfunc(&mut input_offset, &mut input_decoder, &output_info)
+    }
 }
 
 pub fn uu_app() -> Command {
     Command::new(uucore::util_name())
         .version(uucore::crate_version!())
-        .about(ABOUT)
-        .override_usage(format_usage(USAGE))
-        .after_help(AFTER_HELP)
-        .trailing_var_arg(true)
+        .help_template(uucore::localized_help_template(uucore::util_name()))
+        .about(translate!("od-about"))
+        .override_usage(format_usage(&translate!("od-usage")))
+        .after_help(translate!("od-after-help"))
         .dont_delimit_trailing_values(true)
         .infer_long_args(true)
         .args_override_self(true)
@@ -264,34 +277,34 @@ pub fn uu_app() -> Command {
         .arg(
             Arg::new(options::HELP)
                 .long(options::HELP)
-                .help("Print help information.")
-                .action(ArgAction::Help)
+                .help(translate!("od-help-help"))
+                .action(ArgAction::Help),
         )
         .arg(
             Arg::new(options::ADDRESS_RADIX)
                 .short('A')
                 .long(options::ADDRESS_RADIX)
-                .help("Select the base in which file offsets are printed.")
+                .help(translate!("od-help-address-radix"))
                 .value_name("RADIX"),
         )
         .arg(
             Arg::new(options::SKIP_BYTES)
                 .short('j')
                 .long(options::SKIP_BYTES)
-                .help("Skip bytes input bytes before formatting and writing.")
+                .help(translate!("od-help-skip-bytes"))
                 .value_name("BYTES"),
         )
         .arg(
             Arg::new(options::READ_BYTES)
                 .short('N')
                 .long(options::READ_BYTES)
-                .help("limit dump to BYTES input bytes")
+                .help(translate!("od-help-read-bytes"))
                 .value_name("BYTES"),
         )
         .arg(
             Arg::new(options::ENDIAN)
                 .long(options::ENDIAN)
-                .help("byte order to use for multi-byte formats")
+                .help(translate!("od-help-endian"))
                 .value_parser(ShortcutValueParser::new(["big", "little"]))
                 .value_name("big|little"),
         )
@@ -299,132 +312,130 @@ pub fn uu_app() -> Command {
             Arg::new(options::STRINGS)
                 .short('S')
                 .long(options::STRINGS)
-                .help(
-                    "NotImplemented: output strings of at least BYTES graphic chars. 3 is assumed when \
-                     BYTES is not specified.",
-                )
+                .help(translate!("od-help-strings"))
+                .num_args(0..=1)
                 .default_missing_value("3")
                 .value_name("BYTES"),
         )
         .arg(
             Arg::new("a")
                 .short('a')
-                .help("named characters, ignoring high-order bit")
+                .help(translate!("od-help-a"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("b")
                 .short('b')
-                .help("octal bytes")
+                .help(translate!("od-help-b"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("c")
                 .short('c')
-                .help("ASCII characters or backslash escapes")
+                .help(translate!("od-help-c"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("d")
                 .short('d')
-                .help("unsigned decimal 2-byte units")
+                .help(translate!("od-help-d"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("D")
                 .short('D')
-                .help("unsigned decimal 4-byte units")
+                .help(translate!("od-help-d4"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("o")
                 .short('o')
-                .help("octal 2-byte units")
+                .help(translate!("od-help-o"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("I")
                 .short('I')
-                .help("decimal 8-byte units")
+                .help(translate!("od-help-capital-i"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("L")
                 .short('L')
-                .help("decimal 8-byte units")
+                .help(translate!("od-help-capital-l"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("i")
                 .short('i')
-                .help("decimal 4-byte units")
+                .help(translate!("od-help-i"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("l")
                 .short('l')
-                .help("decimal 8-byte units")
+                .help(translate!("od-help-l"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("x")
                 .short('x')
-                .help("hexadecimal 2-byte units")
+                .help(translate!("od-help-x"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("h")
                 .short('h')
-                .help("hexadecimal 2-byte units")
+                .help(translate!("od-help-h"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("O")
                 .short('O')
-                .help("octal 4-byte units")
-                .action(ArgAction::SetTrue)
+                .help(translate!("od-help-capital-o"))
+                .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("s")
                 .short('s')
-                .help("decimal 2-byte units")
+                .help(translate!("od-help-s"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("X")
                 .short('X')
-                .help("hexadecimal 4-byte units")
+                .help(translate!("od-help-capital-x"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("H")
                 .short('H')
-                .help("hexadecimal 4-byte units")
+                .help(translate!("od-help-capital-h"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("e")
                 .short('e')
-                .help("floating point double precision (64-bit) units")
+                .help(translate!("od-help-e"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("f")
                 .short('f')
-                .help("floating point double precision (32-bit) units")
+                .help(translate!("od-help-f"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new("F")
                 .short('F')
-                .help("floating point double precision (64-bit) units")
+                .help(translate!("od-help-capital-f"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::FORMAT)
                 .short('t')
                 .long("format")
-                .help("select output format or formats")
+                .help(translate!("od-help-format"))
                 .action(ArgAction::Append)
                 .num_args(1)
                 .value_name("TYPE"),
@@ -433,17 +444,14 @@ pub fn uu_app() -> Command {
             Arg::new(options::OUTPUT_DUPLICATES)
                 .short('v')
                 .long(options::OUTPUT_DUPLICATES)
-                .help("do not use * to mark line suppression")
+                .help(translate!("od-help-output-duplicates"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
             Arg::new(options::WIDTH)
                 .short('w')
                 .long(options::WIDTH)
-                .help(
-                    "output BYTES bytes per output line. 32 is implied when BYTES is not \
-                     specified.",
-                )
+                .help(translate!("od-help-width"))
                 .default_missing_value("32")
                 .value_name("BYTES")
                 .num_args(..=1),
@@ -451,7 +459,7 @@ pub fn uu_app() -> Command {
         .arg(
             Arg::new(options::TRADITIONAL)
                 .long(options::TRADITIONAL)
-                .help("compatibility mode with one input, offset and label.")
+                .help(translate!("od-help-traditional"))
                 .action(ArgAction::SetTrue),
         )
         .arg(
@@ -527,7 +535,7 @@ where
                 input_offset.print_final_offset();
                 return Err(1.into());
             }
-        };
+        }
     }
 
     if input_decoder.has_error() {
@@ -537,6 +545,105 @@ where
     }
 }
 
+/// Extract and display printable strings from input (od -S option)
+fn extract_strings_from_input(
+    input_strings: &[String],
+    skip_bytes: u64,
+    read_bytes: Option<u64>,
+    min_length: usize,
+    radix: Radix,
+) -> UResult<()> {
+    let inputs = map_input_strings(input_strings);
+    let mut mf = MultifileReader::new(inputs);
+
+    // Apply skip_bytes by reading and discarding
+    let mut skipped = 0u64;
+    while skipped < skip_bytes {
+        let to_skip = std::cmp::min(8192, skip_bytes - skipped);
+        let mut skip_buf = vec![0u8; to_skip as usize];
+        match mf.read(&mut skip_buf) {
+            Ok(0) => break, // EOF reached
+            Ok(n) => skipped += n as u64,
+            Err(_) => break,
+        }
+    }
+
+    // Helper function to format and print a string
+    let print_string = |offset: u64, string: &[u8]| {
+        let string_content = String::from_utf8_lossy(string);
+        match radix {
+            Radix::NoPrefix => println!("{string_content}"),
+            Radix::Decimal => println!("{offset:07} {string_content}"),
+            Radix::Hexadecimal => println!("{offset:07x} {string_content}"),
+            Radix::Octal => println!("{offset:07o} {string_content}"),
+        }
+    };
+
+    let mut current_string = Vec::new();
+    let mut string_start_offset = 0u64;
+    let mut current_offset = skip_bytes;
+    let mut bytes_read = 0u64;
+    let mut buf = [0u8; 1];
+
+    loop {
+        // Check if we've reached the read_bytes limit
+        if let Some(limit) = read_bytes {
+            if bytes_read >= limit {
+                // Special case: when -N limit is reached with a pending string
+                // that meets min_length, output it even without null terminator
+                if current_string.len() >= min_length {
+                    print_string(string_start_offset, &current_string);
+                }
+                break;
+            }
+        }
+
+        // Read one byte at a time
+        match mf.read(&mut buf) {
+            Ok(0) => break, // EOF
+            Ok(_) => {
+                bytes_read += 1;
+                let byte = buf[0];
+
+                // Check if it's a printable character (including space)
+                if (0x20..=0x7E).contains(&byte) {
+                    if current_string.is_empty() {
+                        string_start_offset = current_offset;
+                    }
+                    current_string.push(byte);
+                } else {
+                    // Either null terminator or non-printable character
+                    if byte == 0 && current_string.len() >= min_length {
+                        // Null terminator found with valid string
+                        print_string(string_start_offset, &current_string);
+                    }
+                    current_string.clear();
+                }
+
+                current_offset += 1;
+            }
+            Err(e) => {
+                // Note: GNU od does not output unterminated strings at EOF
+                // Strings must be null-terminated to be output
+                if mf.has_error() {
+                    show_error!("{}", e);
+                    return Err(1.into());
+                }
+                break;
+            }
+        }
+    }
+
+    // GNU od doesn't output an offset when strings mode finds no valid strings
+    // This includes cases with only unterminated or too-short strings
+
+    if mf.has_error() {
+        Err(1.into())
+    } else {
+        Ok(())
+    }
+}
+
 /// Outputs a single line of input, into one or more lines human readable output.
 fn print_bytes(prefix: &str, input_decoder: &MemoryDecoder, output_info: &OutputInfo) {
     let mut first = true; // First line of a multi-format raster.
@@ -562,6 +669,10 @@ fn print_bytes(prefix: &str, input_decoder: &MemoryDecoder, output_info: &Output
                     let p = input_decoder.read_float(b, f.formatter_item_info.byte_size);
                     output_text.push_str(&func(p));
                 }
+                FormatWriter::BFloatWriter(func) => {
+                    let p = input_decoder.read_bfloat(b);
+                    output_text.push_str(&func(p));
+                }
                 FormatWriter::MultibyteWriter(func) => {
                     output_text.push_str(&func(input_decoder.get_full_buffer(b)));
                 }
@@ -596,6 +707,17 @@ fn print_bytes(prefix: &str, input_decoder: &MemoryDecoder, output_info: &Output
     }
 }
 
+/// Helper function to convert input strings to InputSource
+fn map_input_strings(input_strings: &[String]) -> Vec<InputSource<'_>> {
+    input_strings
+        .iter()
+        .map(|w| match w as &str {
+            "-" => InputSource::Stdin,
+            x => InputSource::FileName(x),
+        })
+        .collect()
+}
+
 /// returns a reader implementing `PeekRead + Read + HasError` providing the combined input
 ///
 /// `skip_bytes` is the number of bytes skipped from the input
@@ -604,16 +726,9 @@ fn open_input_peek_reader(
     input_strings: &[String],
     skip_bytes: u64,
     read_bytes: Option<u64>,
-) -> PeekReader<BufReader<PartialReader<MultifileReader>>> {
+) -> PeekReader<BufReader<PartialReader<MultifileReader<'_>>>> {
     // should return  "impl PeekRead + Read + HasError" when supported in (stable) rust
-    let inputs = input_strings
-        .iter()
-        .map(|w| match w as &str {
-            "-" => InputSource::Stdin,
-            x => InputSource::FileName(x),
-        })
-        .collect::<Vec<_>>();
-
+    let inputs = map_input_strings(input_strings);
     let mf = MultifileReader::new(inputs);
     let pr = PartialReader::new(mf, skip_bytes, read_bytes);
     // Add a BufReader over the top of the PartialReader. This will have the
@@ -635,11 +750,13 @@ fn format_error_message(error: &ParseSizeError, s: &str, option: &str) -> String
     // GNU's od echos affected flag, -N or --read-bytes (-j or --skip-bytes, etc.), depending user's selection
     match error {
         ParseSizeError::InvalidSuffix(_) => {
-            format!("invalid suffix in --{option} argument {}", s.quote())
+            translate!("od-error-invalid-suffix", "option" => option, "value" => s.quote())
         }
         ParseSizeError::ParseFailure(_) | ParseSizeError::PhysicalMem(_) => {
-            format!("invalid --{option} argument {}", s.quote())
+            translate!("od-error-invalid-argument", "option" => option, "value" => s.quote())
+        }
+        ParseSizeError::SizeTooBig(_) => {
+            translate!("od-error-argument-too-large", "option" => option, "value" => s.quote())
         }
-        ParseSizeError::SizeTooBig(_) => format!("--{option} argument {} too large", s.quote()),
     }
 }
