@@ -4,8 +4,6 @@
 // file that was distributed with this source code.
 // spell-checker:ignore lmnop xlmnop
 use uutests::new_ucmd;
-use uutests::util::TestScenario;
-use uutests::util_name;
 
 #[test]
 fn test_invalid_arg() {
@@ -230,6 +228,52 @@ fn test_separator_and_terminator() {
         .stdout_is("2\\n3\\n4\\n5\\n6\n");
 }
 
+#[test]
+#[cfg(target_os = "linux")]
+fn test_separator_non_utf8() {
+    use std::{ffi::OsString, os::unix::ffi::OsStringExt};
+
+    fn create_arg(prefix: &[u8]) -> OsString {
+        let separator = [0xFF, 0xFE];
+        OsString::from_vec([prefix, &separator].concat())
+    }
+
+    let short = create_arg(b"-s");
+    let long = create_arg(b"--separator=");
+    let expected = [b'1', 0xFF, 0xFE, b'2', b'\n'];
+
+    for arg in [short, long] {
+        new_ucmd!()
+            .arg(&arg)
+            .arg("2")
+            .succeeds()
+            .stdout_is_bytes(expected);
+    }
+}
+
+#[test]
+#[cfg(target_os = "linux")]
+fn test_terminator_non_utf8() {
+    use std::{ffi::OsString, os::unix::ffi::OsStringExt};
+
+    fn create_arg(prefix: &[u8]) -> OsString {
+        let terminator = [0xFF, 0xFE];
+        OsString::from_vec([prefix, &terminator].concat())
+    }
+
+    let short = create_arg(b"-t");
+    let long = create_arg(b"--terminator=");
+    let expected = [b'1', b'\n', b'2', 0xFF, 0xFE];
+
+    for arg in [short, long] {
+        new_ucmd!()
+            .arg(&arg)
+            .arg("2")
+            .succeeds()
+            .stdout_is_bytes(expected);
+    }
+}
+
 #[test]
 fn test_equalize_widths() {
     let args = ["-w", "--equal-width"];
@@ -943,7 +987,7 @@ fn test_parse_out_of_bounds_exponents() {
         .stdout_only("-0\n1\n");
 }
 
-#[ignore]
+#[ignore = ""]
 #[test]
 fn test_parse_valid_hexadecimal_float_format_issues() {
     // These tests detect differences in the representation of floating-point values with GNU seq.
